@@ -1,4 +1,4 @@
-from typing import Any, Dict, Optional
+from typing import Any, Optional, Tuple
 
 from enum import IntEnum
 from dataclasses import dataclass
@@ -9,9 +9,10 @@ class Scope(IntEnum):
     TRANSIENT: int = 1
 
 
-@dataclass(frozen=True)
+@dataclass
 class DependencyWrapper:
     type_: Any
     target: Any
-    annotations: Optional[Dict[str, Any]] = None
+    annotations: Optional[Tuple[Tuple[str, Any]]] = None
     scope: int = Scope.TRANSIENT
+    cache: Optional[Any] = None
